package com.ruoyi.ss.room.service.impl;

import java.util.ArrayList;
import java.util.List;

import cn.hutool.core.util.StrUtil;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.ss.device.domain.vo.DeviceVO;
import com.ruoyi.ss.device.service.DeviceService;
import com.ruoyi.ss.equipment.domain.Equipment;
import com.ruoyi.ss.equipment.service.IEquipmentService;
import com.ruoyi.ss.feeRule.domain.FeeRuleVO;
import com.ruoyi.ss.feeRule.service.IFeeRuleService;
import com.ruoyi.ss.room.domain.*;
import com.ruoyi.ss.room.service.IRoomAssembler;
import com.ruoyi.ss.store.service.IStoreService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.ss.room.mapper.RoomMapper;
import com.ruoyi.ss.room.service.IRoomService;
import org.springframework.transaction.support.TransactionTemplate;

import static com.ruoyi.common.constant.ServiceConstants.FACILITY_TYPE_ROOM_DOOR;
import static com.ruoyi.common.constant.ServiceConstants.FACILITY_TYPE_ROOM_LIGHT;

/**
 * 房间Service业务层处理
 *
 * @author qzz
 * @date 2024-11-20
 */
@Service
public class RoomServiceImpl implements IRoomService
{
    @Autowired
    private RoomMapper roomMapper;

    @Autowired
    private DeviceService smDeviceService;

    @Autowired
    private IStoreService storeService;

    @Autowired
    private TransactionTemplate transactionTemplate;

    @Autowired
    private IEquipmentService equipmentService;

    @Autowired
    private IFeeRuleService feeRuleService;


    /**
     * 查询房间
     *
     * @param roomId 房间主键
     * @return 房间
     */
    @Override
    public RoomVO selectERoomByRoomId(Long roomId)
    {
        return roomMapper.selectERoomByRoomId(roomId);
    }
//
//    /**
//     * 查询房间
//     *
//     * @param roomId 房间主键
//     * @return 房间
//     */
//    @Override
//    public RoomVO selectSimpleRoomByRoomId(Long roomId)
//    {
//        return roomMapper.selectERoomByRoomId(roomId);
//    }

    /**
     * 查询房间列表
     *
     * @param eRoom 房间
     * @return 房间
     */
    @Override
    public List<RoomVO> selectERoomList(RoomQuery eRoom)
    {
        return roomMapper.selectERoomList(eRoom);
    }

    /**
     * 根据店铺id查询房间列表
     *
     * @param storeId 店铺id
     * @return 房间集合
     */
    @Override
    public List<RoomVO> selectERoomListByStoreId(Long storeId) {
        return roomMapper.selectERoomListByStoreId(storeId);
    }

    /**
     * 新增房间
     *
     * @param eRoom 房间
     * @return 结果  equipment
     */
    @Override
    public int insertERoom(Room eRoom)
    {
        eRoom.setCreateTime(DateUtils.getNowDate());
        Boolean execute = transactionTemplate.execute(e -> {
            int i = storeService.updateStoreTypeTags(eRoom.getStoreId(), eRoom.getType());
            ServiceUtil.assertion(i == 0, "店铺类型标签更新失败");

            int i1 = roomMapper.insertERoom(eRoom);
            ServiceUtil.assertion(i1 == 0, "新增房间失败");

            // 新建房间时默认新建房间灯和房间门
            int i2 = createEquipment(eRoom,"房间门",FACILITY_TYPE_ROOM_DOOR);
            ServiceUtil.assertion(i2 == 0, "新增房间门失败");

            int i3  = createEquipment(eRoom,"房间灯",FACILITY_TYPE_ROOM_LIGHT);
            ServiceUtil.assertion(i3 == 0, "新增房间门失败");

            return Boolean.TRUE;
        });
        if(Boolean.FALSE.equals(execute))throw new ServiceException("新增房间失败");
        return 1;
    }

    private int createEquipment(Room eRoom,String equipmentName,String type) {
        Equipment equipment = new Equipment();
        equipment.setRoomId(eRoom.getRoomId());
        equipment.setName(equipmentName);
        equipment.setType(type);
        equipment.setStoreId(eRoom.getStoreId());
        equipment.setStoreName(eRoom.getStoreName());
        return equipmentService.insertEquipment(equipment);
    }

    /**
     * 修改房间
     *
     * @param eRoom 房间
     * @return 结果
     */
    @Override
    public int updateERoom(Room eRoom)
    {
        if(eRoom.getStoreId() != null && StrUtil.isNotBlank(eRoom.getType())){
            int i = storeService.updateStoreTypeTags(eRoom.getStoreId(), eRoom.getType());
            ServiceUtil.assertion(i == 0, "店铺类型标签更新失败");
        }
        return roomMapper.updateERoom(eRoom);
    }

    /**
     * 批量删除房间
     *
     * @param roomIds 需要删除的房间主键
     * @return 结果
     */
    @Override
    public int deleteERoomByRoomIds(Long[] roomIds)
    {
        return roomMapper.deleteERoomByRoomIds(roomIds);
    }

    /**
     * 删除房间信息
     *
     * @param roomId 房间主键
     * @return 结果
     */
    @Override
    public int deleteERoomByRoomId(Long roomId)
    {
        return roomMapper.deleteERoomByRoomId(roomId);
    }

    /**
     * 开门
     * @param eRoomVO 房间对象
     */
    @Override
    public boolean openGate(RoomVO eRoomVO) {
        return smDeviceService.switchDevice(eRoomVO.getDoorId(), true);
    }

    /**
     * 根据店铺查房间
     * @param storeId 店铺id
     * @return String
     */
    @Override
    public List<RoomVO> getRoomListByStoreId(Long storeId) {
        return selectERoomListByStoreId(storeId);
    }

    /**
     * 下架设备
     * @param deviceId 设备id
     * @return String
     */
    @Override
    public int offlineDevice(Long deviceId) {
        return roomMapper.offlineDevice(deviceId);
    }

    /**
     * 绑定房间门
     * @param dto 房间对象
     * @return String
     */
    @Override
    public int bandDoor(RoomDTO dto) {
        // 根据sn查询设备
        DeviceVO deviceVO = smDeviceService.selectBySn(dto.getSn());
        ServiceUtil.assertion(deviceVO == null, "设备不存在");
        // 更新房间门sn
        Room room = new Room();
        room.setRoomId(dto.getRoomId());
        room.setDoorId(deviceVO.getDeviceId());
        return roomMapper.updateERoom(room);
    }

    /**
     * 添加大堂设备
     * @param room 房间对象
     * @return String
     */
    @Override
    public int addLobbyDevice(Room room) {
        return roomMapper.insertERoom(room);
    }


    /**
     * 应用套餐
     */
    @Override
    public int bandRule(BandRuleDTO dto) {
        RoomVO roomVO = roomMapper.selectERoomByRoomId(dto.getRoomId());
        ServiceUtil.assertion(roomVO == null,"房间不存在");

        Long ruleId = dto.getRuleId();
        List<Long> ruleIds = roomVO.getRuleIds();

        if(ruleIds.isEmpty()){
            ruleIds = new ArrayList<>();
        }

        // 检查是否已包含 deviceId，未包含则添加
        if (ruleIds.contains(ruleId)) {
            return 1;
        }

        List<Long> newDevices = new ArrayList<>(ruleIds);
        newDevices.add(ruleId);

        // 更新房间
        Room room = new Room();
        room.setRuleIds(newDevices);
        room.setRoomId(dto.getRoomId());
        int i = updateERoom(room);
        ServiceUtil.assertion(i == 0, "更新房间失败");
        return i;
    }

    /**
     * 解除套餐
     */
    @Override
    public int unBandRule(Long ruleId,Long roomId) {
        // todo 解绑前校验
        int i = roomMapper.unBandRule(ruleId,roomId);
        ServiceUtil.assertion(i ==0, "解除套餐失败");
        return i;
    }
}
